@@ -137,6 +137,7 @@ CancelConnect(int signum)
 int
 OpenControlConnection(const FTPCIPtr cip, char *host, unsigned int port)
 {
+	struct sockaddr_in localAddr;
 	struct in_addr ip_address;
 	int err = 0;
 	int result;
@@ -221,6 +222,19 @@ OpenControlConnection(const FTPCIPtr cip, char *host, unsigned int port)
 			return (kErrNewStreamSocket);
 		}
 
+		/* On rare occasions, the user may specify a local IP address to use. */
+		if (cip->preferredLocalAddr.sin_family != 0) {
+			localAddr = cip->preferredLocalAddr;
+			localAddr.sin_port = 0;
+			if (BindToEphemeralPortNumber(sockfd, &localAddr, (int) cip->ephemLo, (int) cip->ephemHi) < 0) {
+				FTPLogError(cip, kDoPerror, "Could not bind the control socket");
+				result = kErrBindCtrlSocket;
+				cip->errNo = kErrBindCtrlSocket;
+				(void) SClose(sockfd, 3);
+				return (kErrBindCtrlSocket);
+			}
+		}
+
 		/* This doesn't do anything if you left these
 		 * at their defaults (zero).  Otherwise it
 		 * tries to set the buffer size to the
@@ -306,6 +320,19 @@ OpenControlConnection(const FTPCIPtr cip, char *host, unsigned int port)
 			 */
 			(void) memcpy(&cip->servCtlAddr.sin_addr, *curaddr, (size_t) hp.h_length);
 
+			/* On rare occasions, the user may specify a local IP address to use. */
+			if (cip->preferredLocalAddr.sin_family != 0) {
+				localAddr = cip->preferredLocalAddr;
+				localAddr.sin_port = 0;
+				if (BindToEphemeralPortNumber(sockfd, &localAddr, (int) cip->ephemLo, (int) cip->ephemHi) < 0) {
+					FTPLogError(cip, kDoPerror, "Could not bind the control socket");
+					result = kErrBindCtrlSocket;
+					cip->errNo = kErrBindCtrlSocket;
+					(void) SClose(sockfd, 3);
+					return (kErrBindCtrlSocket);
+				}
+			}
+
 			/* This doesn't do anything if you left these
 			 * at their defaults (zero).  Otherwise it
 			 * tries to set the buffer size to the
@@ -891,8 +918,8 @@ FTPFixClientDataAddr(const FTPCIPtr cip)
 
 
 
-static int
-BindToEphemeralPortNumber(int sockfd, struct sockaddr_in *addrp, int ephemLo, int ephemHi)
+int
+BindToEphemeralPortNumber(const int sockfd, struct sockaddr_in *const addrp, const int ephemLo, const int ephemHi)
 {
 	int i;
 	int result;
@@ -919,7 +946,7 @@ BindToEphemeralPortNumber(int sockfd, struct sockaddr_in *addrp, int ephemLo, in
 			if (result == 0)
 				break;
 			sleep(1);
-			if ((errno != 999)
+			if ((errno == 999)
 				/* This next line is just fodder to
 				 * shut the compiler up about variable
 				 * not being used.
@@ -984,6 +1011,9 @@ tryPort2:
 
 	if ((cip->hasPASV == kCommandNotAvailable) || (mode == kSendPortMode)) {
 tryPort:
+		/* Use the same IP that we're using for our control connection,
+		 * which may have been set to something explicitly.
+		 */
 		cip->ourDataAddr = cip->ourCtlAddr;
 		cip->ourDataAddr.sin_family = AF_INET;
 
