@@ -465,6 +465,8 @@ FTPQueryFeatures(const FTPCIPtr cip)
 		cip->hasMLSD = kCommandNotAvailable;
 		cip->hasSITE_UTIME = kCommandNotAvailable;
 		cip->hasHELP_SITE = kCommandNotAvailable;
+		cip->hasMFMT = kCommandNotAvailable;
+		cip->hasMFF = kCommandNotAvailable;
 	}
 	
 	rp = InitResponse();
@@ -504,6 +506,8 @@ FTPQueryFeatures(const FTPCIPtr cip)
 			 */
 			cip->hasMLST = kCommandNotAvailable;
 			cip->hasMLSD = kCommandNotAvailable;
+			cip->hasMFMT = kCommandNotAvailable;
+			cip->hasMFF = kCommandNotAvailable;
 		} else {
 			cip->hasFEAT = kCommandAvailable;
 
@@ -526,6 +530,10 @@ FTPQueryFeatures(const FTPCIPtr cip)
 					cip->hasSIZE = kCommandAvailable;
 				} else if (ISTRNCMP(cp, "MDTM", 4) == 0) {
 					cip->hasMDTM = kCommandAvailable;
+				} else if (ISTRNCMP(cp, "MFMT", 4) == 0) {
+					cip->hasMFMT = kCommandAvailable;
+				} else if (ISTRNCMP(cp, "MFF", 3) == 0) {
+					cip->hasMFF = kCommandAvailable;
 				} else if (ISTRNCMP(cp, "REST", 4) == 0) {
 					cip->hasREST = kCommandAvailable;
 				} else if (ISTRNCMP(cp, "UTIME", 5) == 0) {
@@ -1000,6 +1008,8 @@ FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf,
 	cip->hasSIZE = kCommandAvailabilityUnknown;
 	cip->hasMDTM = kCommandAvailabilityUnknown;
 	cip->hasMDTM_set = kCommandAvailabilityUnknown;
+	cip->hasMFMT = kCommandAvailabilityUnknown;
+	cip->hasMFF = kCommandAvailabilityUnknown;
 	cip->hasREST = kCommandAvailabilityUnknown;
 	cip->hasNLST_a = kCommandAvailabilityUnknown;
 	cip->hasNLST_d = kCommandAvailabilityUnknown;
