@@ -71,6 +71,7 @@ Usage(void)
   -z/-Z  Do (do not) try to resume downloads (default: -z).\n\
   -E     Use regular (PORT) data connections.\n\
   -F     Use passive (PASV) data connections (default).\n\
+  -I XX  Use IP address XX for local source address.\n\
   -DD    Delete remote file after successfully downloading it.\n\
   -b     Run in background (submit job to \"ncftpbatch\" and run).\n\
   -bb    Same as \"-b\" but queue only (do not run \"ncftpbatch\").\n");
@@ -187,6 +188,7 @@ main(int argc, char **argv)
 	int i;
 	char *urlfilep;
 	const char *urldirp;
+	const char *preferredLocalAddrStr = NULL;
 	char precmd[320], postcmd[320], perfilecmd[320];
 	GetoptInfo opt;
 
@@ -228,7 +230,7 @@ main(int argc, char **argv)
 	perfilecmd[0] = '\0';
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:ADzZEFbcCB:W:X:Y:")) > 0) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:ADzZEFbcCB:W:X:Y:I:")) > 0) {
 		if (c == 'b') {
 			batchmode++;
 		}
@@ -236,7 +238,7 @@ main(int argc, char **argv)
 
 	if (batchmode > 0) {
 		GetoptReset(&opt);
-		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:I:")) > 0) switch(c) {
 			case 'v': case 'V': case 'A': case 'B': case 'T':
 			case 'd': case 'e': case 't': case 'r': case 'c':
 			case 'z': case 'Z': case 'C':
@@ -247,7 +249,7 @@ main(int argc, char **argv)
 	}
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:")) > 0) switch(c) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:I:")) > 0) switch(c) {
 		case 'P':
 			fi.port = atoi(opt.arg);	
 			break;
@@ -281,8 +283,10 @@ main(int argc, char **argv)
 				fi.errLog = stdout;
 			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.errLog = stderr;
-			else
+			else {
 				fi.errLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
+				fi.debugTimestamping = 2;
+			}
 			break;
 		case 'D':
 			/* Require two -D's in case they typo. */
@@ -295,8 +299,10 @@ main(int argc, char **argv)
 				fi.debugLog = stdout;
 			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.debugLog = stderr;
-			else
+			else {
 				fi.debugLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
+				fi.debugTimestamping = 2;
+			}
 			break;
 		case 't':
 			SetTimeouts(&fi, opt.arg);
@@ -364,6 +370,13 @@ main(int argc, char **argv)
 		case 'C':
 			ftpcat = 2;
 			break;
+		case 'I':
+			if (AddrStrToAddr(opt.arg, &fi.preferredLocalAddr, 21) < 0) {
+				fprintf(stderr, "Bad IP address (\"%s\") used with -I.\n", opt.arg);
+				Usage();
+			}
+			preferredLocalAddrStr = opt.arg;
+			break;
 		default:
 			Usage();
 	}
@@ -526,6 +539,7 @@ main(int argc, char **argv)
 				(time_t) 0,	/* when: now */
 				0,
 				fi.manualOverrideFeatures,
+				preferredLocalAddrStr,
 				0
 			);
 			if ((rc == 0) && (batchmode < 3)) {
@@ -571,6 +585,7 @@ main(int argc, char **argv)
 					(time_t) 0,	/* when: now */
 					0,
 					fi.manualOverrideFeatures,
+					preferredLocalAddrStr,
 					0
 				);
 				if ((rc == 0) && (batchmode < 3)) {
