@@ -78,6 +78,7 @@ Usage(void)
   -bb    Same as \"-b\" but queue only (do not run \"ncftpbatch\").\n\
   -E     Use regular (PORT) data connections.\n\
   -F     Use passive (PASV) data connections (default).\n\
+  -I XX  Use IP address XX for local source address.\n\
   -y     Try using \"SITE UTIME\" to preserve timestamps on remote host.\n");
 	(void) fprintf(fp, "\
   -B XX  Try setting the SO_SNDBUF size to XX.\n\
@@ -176,6 +177,7 @@ main(int argc, char **argv)
 	char *Umask = NULL;
 	char *dstdir = NULL;
 	char *dstfile = NULL;
+	const char *preferredLocalAddrStr = NULL;
 	char *lfile = NULL;
 	char **files = (char **) 0;
 	int progmeters;
@@ -230,7 +232,7 @@ main(int argc, char **argv)
 	perfilecmd[0] = '\0';
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:I:")) > 0) {
 		if (c == 'b') {
 			batchmode++;
 		}
@@ -238,7 +240,7 @@ main(int argc, char **argv)
 
 	if (batchmode > 0) {
 		GetoptReset(&opt);
-		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:AT:S:EFcCyZzDbB:W:X:Y:I:")) > 0) switch(c) {
 			case 'v': case 'V': case 'A': case 'B': case 'S':
 			case 'T': case 'd': case 'e': case 'U': case 't':
 			case 'm': case 'r': case 'c': case 'y': case 'z':
@@ -250,7 +252,7 @@ main(int argc, char **argv)
 	}
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:")) > 0) switch(c) {
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:I:")) > 0) switch(c) {
 		case 'P':
 			fi.port = atoi(opt.arg);	
 			break;
@@ -284,8 +286,10 @@ main(int argc, char **argv)
 				fi.errLog = stdout;
 			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.errLog = stderr;
-			else
+			else {
 				fi.errLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
+				fi.debugTimestamping = 2;
+			}
 			break;
 		case 'D':
 			/* Require two -D's in case they typo. */
@@ -298,8 +302,10 @@ main(int argc, char **argv)
 				fi.debugLog = stdout;
 			else if (strcmp(opt.arg, "stderr") == 0)
 				fi.debugLog = stderr;
-			else
+			else {
 				fi.debugLog = fopen(opt.arg, FOPEN_APPEND_TEXT);
+				fi.debugTimestamping = 2;
+			}
 			break;
 		case 'U':
 			Umask = opt.arg;
@@ -381,6 +387,13 @@ main(int argc, char **argv)
 			STRNCAT(postcmd, opt.arg);
 			STRNCAT(postcmd, "\n");
 			break;
+		case 'I':
+			if (AddrStrToAddr(opt.arg, &fi.preferredLocalAddr, 21) < 0) {
+				fprintf(stderr, "Bad IP address (\"%s\") used with -I.\n", opt.arg);
+				Usage();
+			}
+			preferredLocalAddrStr = opt.arg;
+			break;
 		default:
 			Usage();
 	}
@@ -516,6 +529,7 @@ main(int argc, char **argv)
 				(time_t) 0,	/* when: now */
 				0,
 				fi.manualOverrideFeatures,
+				preferredLocalAddrStr,
 				0
 			);
 			if ((result == 0) && (batchmode < 3)) {
